@@ -11,7 +11,7 @@ from .utils import urlsafe_base64_encode
 
 
 # 上传策略，参数规格详见
-# http://developer.qiniu.com/docs/v6/api/reference/security/put-policy.html
+# https://developer.qiniu.com/kodo/manual/1206/put-policy
 _policy_fields = set([
     'callbackUrl',       # 回调URL
     'callbackBody',      # 回调Body
@@ -35,10 +35,8 @@ _policy_fields = set([
     'persistentNotifyUrl',  # 持久化处理结果通知URL
     'persistentPipeline',   # 持久化处理独享队列
     'deleteAfterDays',      # 文件多少天后自动删除
-])
-
-_deprecated_policy_fields = set([
-    'asyncOps'
+    'fileType',             # 文件的存储类型，0为普通存储，1为低频存储
+    'isPrefixalScope'       # 指定上传文件必须使用的前缀
 ])
 
 
@@ -48,8 +46,8 @@ class Auth(object):
     该类主要内容是七牛上传凭证、下载凭证、管理凭证三种凭证的签名接口的实现，以及回调验证。
 
     Attributes:
-        __access_key: 账号密钥对中的accessKey，详见 https://portal.qiniu.com/setting/key
-        __secret_key: 账号密钥对重的secretKey，详见 https://portal.qiniu.com/setting/key
+        __access_key: 账号密钥对中的accessKey，详见 https://portal.qiniu.com/user/key
+        __secret_key: 账号密钥对重的secretKey，详见 https://portal.qiniu.com/user/key
     """
 
     def __init__(self, access_key, secret_key):
@@ -178,8 +176,6 @@ class Auth(object):
     @staticmethod
     def __copy_policy(policy, to, strict_policy):
         for k, v in policy.items():
-            if k in _deprecated_policy_fields:
-                raise ValueError(k + ' has deprecated')
             if (not strict_policy) or k in _policy_fields:
                 to[k] = v
 
@@ -189,10 +185,93 @@ class RequestsAuth(AuthBase):
         self.auth = auth
 
     def __call__(self, r):
-        token = None
         if r.body is not None and r.headers['Content-Type'] == 'application/x-www-form-urlencoded':
             token = self.auth.token_of_request(r.url, r.body, 'application/x-www-form-urlencoded')
         else:
             token = self.auth.token_of_request(r.url)
         r.headers['Authorization'] = 'QBox {0}'.format(token)
         return r
+
+
+class QiniuMacAuth(object):
+    """
+    Sign Requests
+
+    Attributes:
+        __access_key
+        __secret_key
+
+    http://kirk-docs.qiniu.com/apidocs/#TOC_325b437b89e8465e62e958cccc25c63f
+    """
+
+    def __init__(self, access_key, secret_key):
+        self.qiniu_header_prefix = "X-Qiniu-"
+        self.__checkKey(access_key, secret_key)
+        self.__access_key = access_key
+        self.__secret_key = b(secret_key)
+
+    def __token(self, data):
+        data = b(data)
+        hashed = hmac.new(self.__secret_key, data, sha1)
+        return urlsafe_base64_encode(hashed.digest())
+
+    def token_of_request(self, method, host, url, qheaders, content_type=None, body=None):
+        """
+        <Method> <PathWithRawQuery>
+        Host: <Host>
+        Content-Type: <ContentType>
+        [<X-Qiniu-*> Headers]
+
+        [<Body>] #这里的 <Body> 只有在 <ContentType> 存在且不为 application/octet-stream 时才签进去。
+
+        """
+        parsed_url = urlparse(url)
+        netloc = parsed_url.netloc
+        path = parsed_url.path
+        query = parsed_url.query
+
+        if not host:
+            host = netloc
+
+        path_with_query = path
+        if query != '':
+            path_with_query = ''.join([path_with_query, '?', query])
+        data = ''.join(["%s %s" % (method, path_with_query), "\n", "Host: %s" % host, "\n"])
+
+        if content_type:
+            data += "Content-Type: %s" % (content_type) + "\n"
+
+        data += qheaders
+        data += "\n"
+
+        if content_type and content_type != "application/octet-stream" and body:
+            data += body
+
+        return '{0}:{1}'.format(self.__access_key, self.__token(data))
+
+    def qiniu_headers(self, headers):
+        res = ""
+        for key in headers:
+            if key.startswith(self.qiniu_header_prefix):
+                res += key+": %s\n" % (headers.get(key))
+        return res
+
+    @staticmethod
+    def __checkKey(access_key, secret_key):
+        if not (access_key and secret_key):
+            raise ValueError('QiniuMacAuthSign : Invalid key')
+
+
+class QiniuMacRequestsAuth(AuthBase):
+    def __init__(self, auth):
+        self.auth = auth
+
+    def __call__(self, r):
+        token = self.auth.token_of_request(
+            r.method, r.headers.get('Host', None),
+            r.url, self.auth.qiniu_headers(r.headers),
+            r.headers.get('Content-Type', None),
+            r.body
+            )
+        r.headers['Authorization'] = 'Qiniu {0}'.format(token)
+        return r
